/* GmbSprite - a widget for Gnome Memory Blocks
 * (http://gnome-memblocks.sourceforge.net)
 *
 * Copyright (C) 2000 The Free Software Foundation
 *
 * Author: Volodymyr Babin <vb@dwuj.ichf.edu.pl>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>
#include "gmb-sprite-pixbuf.h"

static GmbSpriteClass * parent_class = NULL;

enum
{
  OPERATION_FINISHED,
  LAST_SIGNAL
};

static guint gmb_sprite_pixbuf_signals[LAST_SIGNAL] = {0};

static void
gmb_sprite_pixbuf_destroy (GtkObject * object)
{
  GmbSpritePixbuf * pixbuf;

  g_return_if_fail (object != NULL);
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (object));

  pixbuf = (GmbSpritePixbuf *) object;

  if (pixbuf->timeout_id)
    {
      gtk_timeout_remove (pixbuf->timeout_id);
      pixbuf->timeout_id = 0;
    }

  if (pixbuf->next_pixbuf)
    {
      gdk_pixbuf_unref (pixbuf->next_pixbuf);
      pixbuf->next_pixbuf = NULL;
    }

  if (pixbuf->pixbuf)
    {
      gdk_pixbuf_unref (pixbuf->pixbuf);
      pixbuf->pixbuf = NULL;
    }

  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
gmb_sprite_pixbuf_size_request (GmbSprite * sprite, guint16 * width, guint16 * height)
{
  GmbSpritePixbuf * pixbuf;
  guint cwidth = 0;
  guint cheight = 0;
  guint nwidth = 0;
  guint nheight = 0;

  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (sprite));

  pixbuf = (GmbSpritePixbuf *) sprite;

  if (pixbuf->pixbuf)
    {
      cwidth = gdk_pixbuf_get_width (pixbuf->pixbuf);
      cheight = gdk_pixbuf_get_height (pixbuf->pixbuf);
    }

  if (pixbuf->next_pixbuf)
    {
      nwidth = gdk_pixbuf_get_width (pixbuf->next_pixbuf);
      nheight = gdk_pixbuf_get_height (pixbuf->next_pixbuf);
    }

  *width = MAX (cwidth, nwidth);
  *height = MAX (cheight, nheight);
}

static void
gmb_sprite_pixbuf_paint (GmbSprite * sprite, GdkDrawable * buffer, GdkRectangle * area, GdkRectangle * buffer_area)
{
  GdkPixbuf * tmp;
  GmbSpritePixbuf * pixbuf;

  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (sprite));

  pixbuf = (GmbSpritePixbuf *) sprite;
  if (pixbuf->pixbuf == NULL && pixbuf->next_pixbuf == NULL)
    return;

  tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, area->width, area->height);
  gdk_pixbuf_get_from_drawable (tmp,
				buffer,
			        gdk_rgb_get_cmap (),
				area->x - buffer_area->x,
	    			area->y - buffer_area->y,
				0, 0,
				area->width,
				area->height);

  if (pixbuf->next_pixbuf)
    gdk_pixbuf_composite (pixbuf->next_pixbuf,
			  tmp,
			  0, 0,
			  MIN (area->width, gdk_pixbuf_get_width (pixbuf->next_pixbuf)),
			  MIN (area->height, gdk_pixbuf_get_height (pixbuf->next_pixbuf)),
			  (gdouble) (sprite->area.x - area->x + 1),
			  (gdouble) (sprite->area.y - area->y + 1),
			  1.0, 1.0,
			  GDK_INTERP_TILES,
			  pixbuf->current_alpha);

  if (pixbuf->pixbuf)
    gdk_pixbuf_composite (pixbuf->pixbuf,
			  tmp,
			  0, 0,
			  MIN (area->width, gdk_pixbuf_get_width (pixbuf->pixbuf)),
			  MIN (area->height, gdk_pixbuf_get_height (pixbuf->pixbuf)),
			  (gdouble) (sprite->area.x - area->x + 1),
			  (gdouble) (sprite->area.y - area->y + 1),
			  1.0, 1.0,
			  GDK_INTERP_TILES,
			  pixbuf->redraw_alpha - pixbuf->current_alpha);

  gdk_pixbuf_render_to_drawable_alpha (tmp,
				       buffer,
				       0, 0,
				       area->x - buffer_area->x,
	    			       area->y - buffer_area->y,
				       area->width, area->height,
				       GDK_PIXBUF_ALPHA_BILEVEL, 127,
				       GDK_RGB_DITHER_NORMAL, 0, 0);
  gdk_pixbuf_unref (tmp);
}

static void
gmb_sprite_pixbuf_finish_operation (GmbSpritePixbuf * sprite)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (sprite));

  sprite->current_alpha = 0;
  sprite->timeout_id = 0;

  if (sprite->pixbuf)
    gdk_pixbuf_unref (sprite->pixbuf);

  sprite->pixbuf = sprite->next_pixbuf;
  sprite->next_pixbuf = NULL;

  gmb_sprite_request_resize ((GmbSprite *) sprite);
  gtk_object_ref (GTK_OBJECT (sprite));
  gtk_signal_emit (GTK_OBJECT (sprite), gmb_sprite_pixbuf_signals[OPERATION_FINISHED]);
  gtk_object_unref (GTK_OBJECT (sprite));
}

static gint
gmb_sprite_pixbuf_smoother (GmbSpritePixbuf * sprite)
{
  sprite->current_alpha += sprite->alpha_step;
  sprite->current_alpha = MIN (sprite->current_alpha, sprite->redraw_alpha);

  if (sprite->current_alpha == sprite->redraw_alpha)
    gmb_sprite_pixbuf_finish_operation (sprite);
  else
    {
      if (((GmbSprite *) sprite)->host)
        gmb_sprite_area_request_update (((GmbSprite *) sprite)->host,
					&((GmbSprite *) sprite)->area);
      sprite->timeout_id = gtk_timeout_add (sprite->step_delay,
					    (GtkFunction) gmb_sprite_pixbuf_smoother,
					    sprite);
    }
  return 0;
}

static void
gmb_sprite_pixbuf_init (GmbSpritePixbuf * pixbuf)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (pixbuf));

  pixbuf->redraw_alpha = 255;
}

static void
gmb_sprite_pixbuf_class_init (GmbSpritePixbufClass * pixbuf_class)
{
  GtkObjectClass * object_class;
  GmbSpriteClass * sprite_class;

  g_return_if_fail (IS_GMB_SPRITE_PIXBUF_CLASS (pixbuf_class));

  parent_class = gtk_type_class (GMB_SPRITE_TYPE);

  object_class = (GtkObjectClass *) pixbuf_class;
  object_class->destroy = gmb_sprite_pixbuf_destroy;

  sprite_class = (GmbSpriteClass *) pixbuf_class;
  sprite_class->size_request = gmb_sprite_pixbuf_size_request;
  sprite_class->paint = gmb_sprite_pixbuf_paint;

  pixbuf_class->operation_finished = NULL;

  gmb_sprite_pixbuf_signals[OPERATION_FINISHED] =
				gtk_signal_new ("operation_finished",
				GTK_RUN_FIRST,
				object_class->type,
				GTK_SIGNAL_OFFSET (GmbSpritePixbufClass, operation_finished),
				gtk_marshal_NONE__NONE,
				GTK_TYPE_NONE, 0);
  gtk_object_class_add_signals (object_class,
				gmb_sprite_pixbuf_signals,
				LAST_SIGNAL);
}

GtkType
gmb_sprite_pixbuf_get_type (void)
{
  static GtkType type = 0;

  if (!type)
    {
      static const GtkTypeInfo type_info =
      {
	"GmbSpritePixbuf",
	sizeof (GmbSpritePixbuf),
	sizeof (GmbSpritePixbufClass),
	(GtkClassInitFunc) gmb_sprite_pixbuf_class_init,
	(GtkObjectInitFunc) gmb_sprite_pixbuf_init,
	NULL, NULL,
	(GtkClassInitFunc) NULL
      };
      type = gtk_type_unique (GMB_SPRITE_TYPE, &type_info);
    }
  return type;
}

GmbSprite *
gmb_sprite_pixbuf_new (void)
{
  return GMB_SPRITE (gtk_type_new (gmb_sprite_pixbuf_get_type ()));
}

void
gmb_sprite_pixbuf_set_pixbuf_now (GmbSpritePixbuf * sprite, GdkPixbuf * pixbuf)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (sprite));

  if (sprite->timeout_id)
    {
      if (sprite->pixbuf == pixbuf)
        {
	  gmb_sprite_pixbuf_abort_operation (sprite);
	  return;
	}
      else if (sprite->next_pixbuf == pixbuf)
        {
	  gmb_sprite_pixbuf_force_operation (sprite);
	  return;
	}
      else
        {
	  gtk_timeout_remove (sprite->timeout_id);
	  sprite->timeout_id = 0;

	  if (sprite->next_pixbuf)
	    gdk_pixbuf_unref (sprite->next_pixbuf);
	  sprite->next_pixbuf = NULL;
	}
    }

  if (sprite->pixbuf != pixbuf)
    {
      if (pixbuf)
        gdk_pixbuf_ref (pixbuf);

      if (sprite->pixbuf)
        gdk_pixbuf_unref (sprite->pixbuf);

      sprite->pixbuf = pixbuf;
      gmb_sprite_request_resize ((GmbSprite *) sprite);
    }
}

void
gmb_sprite_pixbuf_set_pixbuf_smooth (GmbSpritePixbuf * sprite,
				     GdkPixbuf * pixbuf,
				     guint alpha_step,
				     guint step_delay)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (sprite));

  if (sprite->pixbuf == pixbuf || (sprite->timeout_id && sprite->next_pixbuf == pixbuf))
    return;
  else
    {
      sprite->alpha_step = CLAMP (alpha_step, 1, 255);
      sprite->current_alpha = 0;
      sprite->step_delay = MAX (1, step_delay);

      if (sprite->next_pixbuf)
        gdk_pixbuf_unref (sprite->next_pixbuf);
    
      sprite->next_pixbuf = pixbuf;

      if (pixbuf)
	gdk_pixbuf_ref (pixbuf);

      gmb_sprite_request_resize (GMB_SPRITE (sprite));

      if (sprite->timeout_id == 0)
        sprite->timeout_id = gtk_timeout_add (sprite->step_delay,
					      (GtkFunction) gmb_sprite_pixbuf_smoother,
					      sprite);
      else
        (void) gmb_sprite_pixbuf_smoother (sprite);
    }
}

GdkPixbuf *
gmb_sprite_pixbuf_get_pixbuf (GmbSpritePixbuf * pixbuf)
{
  g_return_val_if_fail (IS_GMB_SPRITE_PIXBUF (pixbuf), NULL);

  return pixbuf->pixbuf;
}

gboolean
gmb_sprite_pixbuf_is_busy (GmbSpritePixbuf * pixbuf)
{
  g_return_val_if_fail (IS_GMB_SPRITE_PIXBUF (pixbuf), FALSE);

  return (pixbuf->timeout_id == 0 ? FALSE : TRUE);
}

void
gmb_sprite_pixbuf_force_operation (GmbSpritePixbuf * pixbuf)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (pixbuf));

  if (pixbuf->timeout_id)
    {
      gtk_timeout_remove (pixbuf->timeout_id);
      gmb_sprite_pixbuf_finish_operation (pixbuf);
    }
}

void
gmb_sprite_pixbuf_abort_operation (GmbSpritePixbuf * pixbuf)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (pixbuf));

  if (pixbuf->timeout_id)
    {
      gtk_timeout_remove (pixbuf->timeout_id);
      pixbuf->timeout_id = 0;
      pixbuf->current_alpha = 0;

      if (pixbuf->next_pixbuf)
        gdk_pixbuf_unref (pixbuf->next_pixbuf);

      pixbuf->next_pixbuf = NULL;
      gmb_sprite_request_resize ((GmbSprite *) pixbuf);
    }
}

void
gmb_sprite_pixbuf_set_alpha (GmbSpritePixbuf * pixbuf, guint alpha)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXBUF (pixbuf));

  if (pixbuf->redraw_alpha != alpha)
    {
      pixbuf->redraw_alpha = CLAMP (alpha, 0, 255);
      if (((GmbSprite *) pixbuf)->host)
        gmb_sprite_area_request_update (((GmbSprite *) pixbuf)->host,
					&((GmbSprite *) pixbuf)->area);
    }
}

guint
gmb_sprite_pixbuf_get_alpha (GmbSpritePixbuf * pixbuf)
{
  g_return_val_if_fail (IS_GMB_SPRITE_PIXBUF (pixbuf), 0);

  return pixbuf->redraw_alpha;
}
