/* GmbSprit - a widget for Gnome Memory Blocks
 * (http://gnome-memblocks.sourceforge.net)
 *
 * Copyright (C) 2000 The Free Software Foundation
 *
 * Authors: Volodymyr Babin <vb@dwuj.ichf.edu.pl>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gmb-sprite-pixmap.h"

static GmbSpriteClass * parent_class = NULL;

static void
gmb_sprite_pixmap_destroy (GtkObject * object)
{
  GmbSpritePixmap * pixmap;

  g_return_if_fail (object != NULL);
  g_return_if_fail (IS_GMB_SPRITE_PIXMAP (object));

  pixmap = (GmbSpritePixmap *) object;

  if (pixmap->pixmap)
    {
      gdk_pixmap_unref (pixmap->pixmap);
      pixmap->pixmap = NULL;
    }

  if (pixmap->mask)
    {
      gdk_bitmap_unref (pixmap->mask);
      pixmap->mask = NULL;
    }

  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
gmb_sprite_pixmap_size_request (GmbSprite * sprite, guint16 * width, guint16 * height)
{
  gint pwidth, pheight;
  GmbSpritePixmap * pixmap;

  g_return_if_fail (IS_GMB_SPRITE_PIXMAP (sprite));

  pixmap = (GmbSpritePixmap *) sprite;

  if (pixmap->pixmap)
    {
      gdk_window_get_size (pixmap->pixmap, &pwidth, &pheight);
      *width = (guint16) pwidth;
      *height = (guint16) pheight;
    }
  else
    {
      *width = 0;
      *height = 0;
    }
}

static void
gmb_sprite_pixmap_paint (GmbSprite * sprite, GdkDrawable * buffer, GdkRectangle * area, GdkRectangle * buffer_area)
{
  GdkGC * gc;
  GmbSpritePixmap * pixmap;

  g_return_if_fail (IS_GMB_SPRITE_PIXMAP (sprite));

  pixmap = (GmbSpritePixmap *) sprite;

  if (!pixmap->pixmap)
    return;

  gc = gdk_gc_new (buffer);

  if (pixmap->mask)
    {
      gdk_gc_set_clip_origin (gc, sprite->area.x - buffer_area->x,
			    	  sprite->area.y - buffer_area->y);
      gdk_gc_set_clip_mask (gc, pixmap->mask);
    }

  gdk_draw_pixmap (buffer, gc, pixmap->pixmap,
		   area->x - sprite->area.x,
		   area->y - sprite->area.y,
		   area->x - buffer_area->x,
	    	   area->y - buffer_area->y,
		   area->width, area->height);

  gdk_gc_unref (gc);
}

static void
gmb_sprite_pixmap_class_init (GmbSpritePixmapClass * pixmap_class)
{
  GtkObjectClass * object_class;
  GmbSpriteClass * sprite_class;

  g_return_if_fail (IS_GMB_SPRITE_PIXMAP_CLASS (pixmap_class));

  parent_class = gtk_type_class (GMB_SPRITE_TYPE);

  object_class = (GtkObjectClass *) pixmap_class;
  object_class->destroy = gmb_sprite_pixmap_destroy;

  sprite_class = (GmbSpriteClass *) pixmap_class;
  sprite_class->size_request = gmb_sprite_pixmap_size_request;
  sprite_class->paint = gmb_sprite_pixmap_paint;
}

GtkType
gmb_sprite_pixmap_get_type (void)
{
  static GtkType type = 0;

  if (!type)
    {
      static const GtkTypeInfo type_info =
      {
	"GmbSpritePixmap",
	sizeof (GmbSpritePixmap),
	sizeof (GmbSpritePixmapClass),
	(GtkClassInitFunc) gmb_sprite_pixmap_class_init,
	(GtkObjectInitFunc) NULL,
	NULL, NULL,
	(GtkClassInitFunc) NULL
      };
      type = gtk_type_unique (GMB_SPRITE_TYPE, &type_info);
    }
  return type;
}

GmbSprite *
gmb_sprite_pixmap_new (void)
{
  return GMB_SPRITE (gtk_type_new (gmb_sprite_pixmap_get_type ()));
}

void
gmb_sprite_pixmap_set_pixmap (GmbSpritePixmap * sprite, GdkPixmap * pixmap, GdkBitmap * mask)
{
  g_return_if_fail (IS_GMB_SPRITE_PIXMAP (sprite));

  if (sprite->pixmap && sprite->pixmap != pixmap)
    gdk_pixmap_unref (sprite->pixmap);
  if (sprite->mask && sprite->mask != mask)
    gdk_bitmap_unref (sprite->mask);
  if (pixmap)
    gdk_pixmap_ref (pixmap);
  if (mask)
    gdk_bitmap_ref (mask);

  sprite->pixmap = pixmap;
  sprite->mask = mask;

  gmb_sprite_request_resize ((GmbSprite *) sprite);
}
