/* test-animator -- test program for gmb-sprite-animator code
 * Copyright (C) 2000 Volodymyr Babin <vb@dwuj.ichf.edu.pl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 *  Extremely stuped code --- it's just test.
 */

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "gmb-sprite/gmb-sprite.h"

static GList * animators = NULL;
static GList * animation = NULL;

static GtkWidget * fsel = NULL;
static GmbSpriteAnimator * last = NULL;
static GtkWidget * gsa;

enum
{
  ANIMATOR_NEW,
  ANIMATOR_DELETE,
  ANIMATOR_RUNNING,
  ANIMATOR_SPEED,
  ANIMATOR_LOOP_NONE,
  ANIMATOR_LOOP_RESTART,
  ANIMATOR_LOOP_PING_PONG,
  ANIMATOR_REVERSED,
  ANIMATOR_ADVANCE,
  LAST_CONTROL
};

static GtkWidget * controls[LAST_CONTROL];

static void
update_controls (void)
{
  if (last)
    {
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (controls[ANIMATOR_SPEED]),
			         gmb_sprite_animator_get_playback_speed (last));
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_REVERSED]),
			         gmb_sprite_animator_is_reversed (last));
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_RUNNING]),
			         gmb_sprite_animator_is_running (last));

      switch (gmb_sprite_animator_get_loop_type (last))
        {
	  case GMB_SPRITE_ANIMATOR_LOOP_NONE :
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_NONE]), TRUE);
	    break;
	  case GMB_SPRITE_ANIMATOR_LOOP_RESTART :
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_RESTART]), TRUE);
	    break;
	  case GMB_SPRITE_ANIMATOR_LOOP_PING_PONG :
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_PING_PONG]), TRUE);
	    break;
	  default :
	    g_assert_not_reached ();
	}
    }
  else
    {
      guint j;

      for (j = 0; j < LAST_CONTROL; j++)
        gtk_widget_set_sensitive (controls[j], FALSE);
    }
}

static void
update_last (void)
{
  if (animators)
    {
      GList * tmp;

      tmp = g_list_last (animators);
      last = GMB_SPRITE_ANIMATOR (tmp->data);
    }
  else
    last = NULL;
}

static void
animation_destroy_cb (GtkObject * a, gpointer data)
{
  if (g_list_length (animation) == 1)
    g_message ("*** Last Opened Animation Being Destroyed ***");

  g_message ("removing destroyed animation from GList");
  animation = g_list_remove (animation, a);
}

static void
fsel_ok (GtkWidget * widget, gpointer data)
{
  gchar * fname;

  gtk_widget_hide (fsel);
  fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fsel));
  if (fname)
    {
      GdkPixbufAnimation * pb_animation;

      pb_animation = gdk_pixbuf_animation_new_from_file (fname);
      if (pb_animation)
        {
	  GmbAnimation * gmb_animation = NULL;

	  g_message ("GdkPixbufAnimation loaded successfully from %s", fname);
	  gmb_animation = gmb_animation_new_from_gdk_pixbuf_animation (pb_animation);
	  gdk_pixbuf_animation_unref (pb_animation);
	  if (gmb_animation)
	    {
	      g_message ("GmbAnimation successfully generated");
	      g_message ("N_FRAMES = %d", gmb_animation_get_n_frames (gmb_animation));
	      animation = g_list_append (animation, gmb_animation);
	      gtk_signal_connect (GTK_OBJECT (gmb_animation), "destroy",
	    			  animation_destroy_cb, NULL);
	      gtk_widget_set_sensitive (controls[ANIMATOR_NEW], TRUE);
	    }
	  else
	    g_warning ("conversion to GmbAnimation failed");
	}
      else
        g_warning ("error loading GdkPixbufAnimation from %s", fname);
    }
  gtk_widget_destroy (fsel);
  fsel = NULL;
}

static void
fsel_cancel (GtkWidget * widget, gpointer data)
{
  gtk_widget_destroy (fsel);
  fsel = NULL;
}

static void
fsel_destroyed (GtkWidget * widget, gpointer data)
{
  fsel = NULL;
}

static void
button_load_cb (GtkWidget * widget, gpointer data)
{
  if (fsel)
    return;

  fsel = gtk_widget_new (GTK_TYPE_FILE_SELECTION,
			 "GtkWindow::title", "Load GIF animation",
			 "GtkWindow::window_position", GTK_WIN_POS_MOUSE, NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fsel)->ok_button), "clicked",
		      GTK_SIGNAL_FUNC (fsel_ok), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fsel)->cancel_button), "clicked",
		      GTK_SIGNAL_FUNC (fsel_cancel), NULL);
  gtk_signal_connect (GTK_OBJECT (fsel), "destroy",
		      GTK_SIGNAL_FUNC (fsel_destroyed), NULL);
  gtk_widget_show (fsel);
}

static void
playback_finished_cb (GmbSpriteAnimator * a, gpointer data)
{
  if (a == last)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_RUNNING]), FALSE);
}

static void
animator_new_cb (GtkWidget * widget, gpointer data)
{
  GList * tmp;
  GmbSpriteAnimator * a;

  if (!animation)
    {
      g_message ("*** You must load something first ***");
      return;
    }

  a = (GmbSpriteAnimator *) gmb_sprite_animator_new ();

  if (!animators)
    {
      guint j;
      for (j = ANIMATOR_DELETE; j < LAST_CONTROL; j++)
        gtk_widget_set_sensitive (controls[j], TRUE);
    }

  animators = g_list_append (animators, a);
  update_last ();

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_NONE])))
    gmb_sprite_animator_set_loop_type (a, GMB_SPRITE_ANIMATOR_LOOP_NONE);
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_RESTART])))
    gmb_sprite_animator_set_loop_type (a, GMB_SPRITE_ANIMATOR_LOOP_RESTART);
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_PING_PONG])))
    gmb_sprite_animator_set_loop_type (a, GMB_SPRITE_ANIMATOR_LOOP_PING_PONG);
  else
    g_assert_not_reached ();

  gmb_sprite_animator_set_playback_speed
    (a, gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (controls[ANIMATOR_SPEED])));
  gmb_sprite_animator_set_reversed
    (a, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_REVERSED])));

  tmp = g_list_last (animation);
  gmb_sprite_animator_set_animation (a, GMB_ANIMATION (tmp->data));

#ifdef TORTURES
  gtk_timeout_add (10000, (GtkFunction) gtk_object_destroy, tmp->data);
#endif

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_RUNNING])))
    gmb_sprite_animator_start_playback (a);

  g_message ("last GmbAnimation ref_count = %d", GTK_OBJECT (tmp->data)->ref_count);

  gtk_signal_connect (GTK_OBJECT (a), "playback_finished",
		      GTK_SIGNAL_FUNC (playback_finished_cb), NULL);

  gmb_sprite_area_add_sprite_top (GMB_SPRITE_AREA (gsa),
				  GMB_SPRITE (a),
				  48 * g_list_length (animators),
				  52 * g_list_length (animators));
}

static void
animator_delete_cb (GtkWidget * widget, gpointer data)
{
  if (last)
    {
      gtk_object_destroy (GTK_OBJECT (last));
      animators = g_list_remove (animators, last);
      update_last ();
      if (last)
        update_controls ();
      else
        {
	  guint j;
	  for (j = ANIMATOR_NEW; j < LAST_CONTROL; j++)
	    gtk_widget_set_sensitive (controls[j], FALSE);
	}
    }
}

static void
running_cb (GtkWidget * w, gpointer data)
{
  if (last)
    {
      gboolean animator_state;
      gboolean toggle_state;

      toggle_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_RUNNING]));
      animator_state = gmb_sprite_animator_is_running (last);

      if (animator_state != toggle_state)
        {
	  if (toggle_state)
	    gmb_sprite_animator_start_playback (last);
	  else
	    gmb_sprite_animator_stop_playback (last);
        }
    }
}

static void
speed_cb (GtkWidget *w, gpointer data)
{
  if (last)
    {
      gdouble speed;
      speed = (gdouble) gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (w));
      gmb_sprite_animator_set_playback_speed (last, speed);
    }
}

static void
loop_none_cb (GtkWidget * w, gpointer data)
{
  if (last)
    {
      if (gmb_sprite_animator_get_loop_type (last) != GMB_SPRITE_ANIMATOR_LOOP_NONE &&
          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_NONE])))
	gmb_sprite_animator_set_loop_type (last, GMB_SPRITE_ANIMATOR_LOOP_NONE);
    }
}

static void
loop_restart_cb (GtkWidget * w, gpointer data)
{
  if (last)
    {
      if (gmb_sprite_animator_get_loop_type (last) != GMB_SPRITE_ANIMATOR_LOOP_RESTART &&
          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_RESTART])))
	gmb_sprite_animator_set_loop_type (last, GMB_SPRITE_ANIMATOR_LOOP_RESTART);
    }
}

static void
loop_ping_pong_cb (GtkWidget * w, gpointer data)
{
  if (last)
    {
      if (gmb_sprite_animator_get_loop_type (last) != GMB_SPRITE_ANIMATOR_LOOP_PING_PONG &&
          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_LOOP_PING_PONG])))
	gmb_sprite_animator_set_loop_type (last, GMB_SPRITE_ANIMATOR_LOOP_PING_PONG);
    }
}

static void
reversed_cb (GtkWidget * w, gpointer data)
{
  if (last)
    {
      gboolean animator_state;
      gboolean toggle_state;

      toggle_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (controls[ANIMATOR_REVERSED]));
      animator_state = gmb_sprite_animator_is_reversed (last);

      if (animator_state != toggle_state)
	gmb_sprite_animator_set_reversed (last, toggle_state);
    }
}

static void
advance_cb (GtkWidget * w, gpointer data)
{
  if (last)
    gmb_sprite_animator_advance (last, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data)));

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (data), 0.0);
}

int
main (int argc, char ** argv)
{
  GtkWidget * window;
  GtkWidget * vbox, * hbox;
  GtkWidget * frame;
  GtkWidget * button_load;
  GtkAdjustment * adj;
  GtkWidget * label;
  GtkWidget * spinner;

  gtk_init (&argc, &argv);
  gtk_widget_set_default_visual (gdk_rgb_get_visual ());
  gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());

  window = gtk_widget_new (GTK_TYPE_WINDOW,
			   "GtkWindow::type", GTK_WINDOW_TOPLEVEL,
			   "GtkWindow::title", "GMB Sprite Animator Test",
			   "GtkContainer::border_width", 2, NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  hbox = gtk_widget_new (GTK_TYPE_HBOX,
			 "GtkBox::spacing", 0,
			 "GtkBox::homogeneous", FALSE, NULL);
  gtk_container_add (GTK_CONTAINER (window), hbox);
  vbox = gtk_widget_new (GTK_TYPE_VBOX,
			 "GtkBox::spacing", 5,
			 "GtkBox::homogeneous", FALSE, NULL);
  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 5);
  frame = gtk_widget_new (GTK_TYPE_FRAME,
			  "GtkFrame::label", " Sprite Area ",
			  "GtkContainer::border_width", 0, NULL);
  gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 5);
  button_load = gtk_button_new_with_label ("Load Animation");
  gtk_signal_connect (GTK_OBJECT (button_load), "clicked",
		      GTK_SIGNAL_FUNC (button_load_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), button_load, FALSE, FALSE, 15);

  controls[ANIMATOR_NEW] = gtk_button_new_with_label ("New Animator");
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_NEW]), "clicked",
		      GTK_SIGNAL_FUNC (animator_new_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), controls[ANIMATOR_NEW], FALSE, FALSE, 0);
  gtk_widget_set_sensitive (controls[ANIMATOR_NEW], FALSE);

  controls[ANIMATOR_DELETE] = gtk_button_new_with_label ("Destroy Animator");
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_DELETE]), "clicked",
		      GTK_SIGNAL_FUNC (animator_delete_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), controls[ANIMATOR_DELETE], FALSE, FALSE, 0);
  gtk_widget_set_sensitive (controls[ANIMATOR_DELETE], FALSE);

  gtk_box_pack_start (GTK_BOX (vbox), gtk_hseparator_new (), FALSE, FALSE, 10);

  controls[ANIMATOR_RUNNING] = gtk_toggle_button_new_with_label ("Running");
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_RUNNING]), "toggled",
		      GTK_SIGNAL_FUNC (running_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), controls[ANIMATOR_RUNNING], FALSE, FALSE, 0);
  gtk_widget_set_sensitive (controls[ANIMATOR_RUNNING], FALSE);
 
  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 15);
  label = gtk_label_new ("Speed");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
  adj = (GtkAdjustment*) gtk_adjustment_new (4.0, 0.0, 32.0, 0.5, 16.0, 16.0);
  controls[ANIMATOR_SPEED] = gtk_spin_button_new (adj, 0, 0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (controls[ANIMATOR_SPEED]), FALSE);
  gtk_spin_button_set_digits (GTK_SPIN_BUTTON (controls[ANIMATOR_SPEED]), 1);
  gtk_box_pack_start (GTK_BOX (hbox), controls[ANIMATOR_SPEED], FALSE, FALSE, 5);
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_SPEED]), "changed",
                      GTK_SIGNAL_FUNC (speed_cb), NULL);
  gtk_widget_set_sensitive (controls[ANIMATOR_SPEED], FALSE);

  controls[ANIMATOR_LOOP_NONE] = gtk_radio_button_new_with_label (NULL, "Loop None");
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_LOOP_NONE]), "toggled",
		      GTK_SIGNAL_FUNC (loop_none_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), controls[ANIMATOR_LOOP_NONE], FALSE, FALSE, 0);
  gtk_widget_set_sensitive (controls[ANIMATOR_LOOP_NONE], FALSE);
  controls[ANIMATOR_LOOP_RESTART] = gtk_radio_button_new_with_label
    (gtk_radio_button_group (GTK_RADIO_BUTTON (controls[ANIMATOR_LOOP_NONE])), "Loop Restart");
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_LOOP_RESTART]), "toggled",
		      GTK_SIGNAL_FUNC (loop_restart_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), controls[ANIMATOR_LOOP_RESTART], FALSE, FALSE, 0);
  gtk_widget_set_sensitive (controls[ANIMATOR_LOOP_RESTART], FALSE);
  controls[ANIMATOR_LOOP_PING_PONG] = gtk_radio_button_new_with_label
    (gtk_radio_button_group (GTK_RADIO_BUTTON (controls[ANIMATOR_LOOP_RESTART])), "Loop Ping-Pong");
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_LOOP_PING_PONG]), "toggled",
		      GTK_SIGNAL_FUNC (loop_ping_pong_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), controls[ANIMATOR_LOOP_PING_PONG], FALSE, FALSE, 0);
  gtk_widget_set_sensitive (controls[ANIMATOR_LOOP_PING_PONG], FALSE);

  controls[ANIMATOR_REVERSED] = gtk_toggle_button_new_with_label ("Reversed");
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_REVERSED]), "toggled",
		      GTK_SIGNAL_FUNC (reversed_cb), NULL);
  gtk_box_pack_start (GTK_BOX (vbox), controls[ANIMATOR_REVERSED], FALSE, FALSE, 5);
  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 15);
  gtk_widget_set_sensitive (controls[ANIMATOR_REVERSED], FALSE);

  controls[ANIMATOR_ADVANCE] = gtk_button_new_with_label ("Advance ");
  gtk_box_pack_start (GTK_BOX (hbox), controls[ANIMATOR_ADVANCE], FALSE, FALSE, 5);
  gtk_widget_set_sensitive (controls[ANIMATOR_ADVANCE], FALSE);
  adj = (GtkAdjustment*) gtk_adjustment_new (0.0, - 16.0, 16.0, 1.0, 16.0, 16.0);
  spinner = gtk_spin_button_new (adj, 0, 0);
  gtk_signal_connect (GTK_OBJECT (controls[ANIMATOR_ADVANCE]), "clicked",
		      GTK_SIGNAL_FUNC (advance_cb), spinner);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), FALSE);
  gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 5);

  gsa = gmb_sprite_area_new ();

  gtk_rc_parse ("tests/test.gtkrc");
  gtk_rc_parse ("../tests/test.gtkrc");
  gtk_rc_parse ("test.gtkrc");

  gtk_container_add (GTK_CONTAINER (frame), gsa);
  gtk_widget_set_usize (gsa, 300, 300);

  gtk_widget_show_all (window);  
  gtk_main ();

  return 0;
}